from datetime import datetime


def noNewOutput(new):
    for a in reversed(new):
        if a.text and not a.text.isspace():
            return False
    return True
# for a in reversed(new):
#     if a.text and not a.text.isspace():
#         break
# else:
#     return


def getOrgDate():
    return datetime.now().strftime("%Y-%m-%d %a %H:%M")



import os.path
from plover.oslayer.config import CONFIG_DIR

fname = os.path.join(CONFIG_DIR, 'clippy_2_cfg.py')
mod = {}

if os.path.isfile(fname):
    with open(fname, encoding='utf-8') as fp:
        source = fp.read()
    exec(source, mod)

Config = mod.get('Config')

if not Config:
    class Config:
        pass

    for key, value in mod.items():
        if key == "__builtins__":
            continue
        else:
            setattr(Config, key, staticmethod(value))



import os.path
from plover.oslayer.config import CONFIG_DIR
from io import IOBase


class State:
    def __init__(self):
        self._output_file_name = None
        self._efficiency_symbol = None
        self._max_pad_efficiency = None
        self._max_pad_english = None
        self._f = None

    @property
    def output_file_name(self):
        return self._output_file_name

    @output_file_name.setter
    def output_file_name(self, val):
        assert type(val) == str and val != ""
        val = os.path.join(CONFIG_DIR, val)
        self._output_file_name = val

    @property
    def efficiency_symbol(self):
        return self._efficiency_symbol

    @efficiency_symbol.setter
    def efficiency_symbol(self, val):
        assert type(val) == str and val != ""
        self._efficiency_symbol = val

    @property
    def max_pad_efficiency(self):
        return self._max_pad_efficiency

    @max_pad_efficiency.setter
    def max_pad_efficiency(self, val):
        assert type(val) == int and val > 0
        self._max_pad_efficiency = val

    @property
    def max_pad_english(self):
        return self._max_pad_english

    @max_pad_english.setter
    def max_pad_english(self, val):
        assert type(val) == int and val > 0
        self._max_pad_english = val

    @property
    def f(self):
        if self._f is None or self._f.closed:
            self.f = open(self.output_file_name, "a")
        return self._f

    @f.setter
    def f(self, val):
        assert isinstance(val, IOBase)
        self._f = val



class Defaults:
    @staticmethod
    def initPre(obj, clippy):
        return obj.org.defaultPre(clippy)

    @staticmethod
    def init(clippy):
        clippy.state.output_file_name = "clippy_2.org"
        clippy.state.efficiency_symbol = "*"
        clippy.state.max_pad_efficiency = 5
        clippy.state.max_pad_english = 15
        clippy.state.last_num_translations = 10

    @staticmethod
    def initPost(obj, clippy):
        pass
        # return obj.org.defaultPost(clippy)

    @staticmethod
    def startPre(obj, clippy):
        return obj.org.defaultPre(clippy)

    @staticmethod
    def startPost(obj, clippy):
        return obj.org.defaultPost(clippy)

    @staticmethod
    def stopPre(obj, clippy):
        return obj.org.defaultPre(clippy)

    @staticmethod
    def stopPost(obj, clippy):
        return obj.org.defaultPost(clippy)

    @staticmethod
    def onTranslatePre(obj, clippy):
        return clippy.translations.org.defaultPre(obj, clippy)

    @staticmethod
    def onTranslateSuggest(obj, clippy):
        return clippy.formatting.org.defaultSuggest(obj, clippy)

    @staticmethod
    def onTranslatePost(obj, clippy):
        return clippy.translations.org.defaultPost(obj, clippy)

    @staticmethod
    def onTranslateFilter(obj, clippy):
        # return clippy.translations.retro.filter(obj, clippy)
        return clippy.translations.filter(obj, clippy)

    @staticmethod
    def onTranslateGenerator(obj, clippy):
        # yield from clippy.translations.retro.generator(obj, clippy)
        yield from clippy.translations.generator(obj, clippy)

    @staticmethod
    def onStrokedPre(obj, clippy):
        return obj.org.defaultPre(obj, clippy)

    @staticmethod
    def onStrokedPost(obj, clippy):
        return obj.org.defaultPost(obj, clippy)



# from .util import noNewOutput
from .state import State
from .default import Defaults
from .actions import Actions
from .translations import Translations
from .formatting import Formatting

from .hooks.initialize import Initialize
from .hooks.start import Start
from .hooks.stop import Stop
from .hooks.translate import OnTranslate
from .hooks.stroke import OnStroked

from plover.engine import StenoEngine
# from plover.translation import Translation


class Clippy:
    def __init__(self, engine: StenoEngine) -> None:
        super().__init__()

        hook = Initialize()
        hook.pre(self)

        self.engine: StenoEngine = engine
        self.state = State()
        self.actions = Actions(self.state)
        self.translations = Translations()
        self.formatting = Formatting()

        Defaults.init(self)

        hook.post(self)

    def start(self) -> None:
        hook = Start()
        hook.pre(self)
        # this order can't be changed ;<
        self.engine.hook_connect('translated', self.onTranslate)
        self.engine.hook_connect('stroked', self.onStroked)
        self.state.f = open(self.state.output_file_name, 'a')

        hook.post(self)

    def stop(self) -> None:
        hook = Stop()
        hook.pre(self)

        self.engine.hook_disconnect('translated', self.onTranslate)
        self.engine.hook_disconnect('stroked', self.onStroked)
        self.state.f.close()

        hook.post(self)

    def onStroked(self, stroke):
        if not self.engine.output:
            return
        hook = OnStroked(stroke)
        hook.pre(self)
        print(self.state.prev_stroke)
        # not sure what else to do here for now
        hook.post(self)

    def onTranslate(self, old, new):
        hook = OnTranslate(old, new)
        hook.pre(self)
        if hook.filter(self):
            for phrase in hook.generator(self):
                self.state.phrase = phrase
                hook.suggest(self)
        hook.post(self)
        # if noNewOutput(new):
        #     return
        # for phrase in self.translations.generator():
        #
        #     (
        #         self.state.english,
        #         self.state.stroked,
        #         self.state.suggestions
        #     ) = phrase
        #     print(f"phrase = {phrase}")
        #
        #     hook.call(self)
        #
        # hook.post(self)



list(tails([1, 2, 3, 4, 5]))
[[5], [4, 5], [3, 4, 5], [2, 3, 4, 5], [1, 2, 3, 4, 5]]


# problem with KA*PD
old = []
new = [Action(text='south', trailing_space=' ', word='south')]
# pass filter
english  = south
stroked  = ['SOUT']
suggestions  = [('SOUT',), ('SO*UT',)]
phrase = ('south', ['SOUT'], [('SOUT',), ('SO*UT',)])
english  = south
translations  = [Translation(('PHROLG',) : "{PLOVER:TOGGLE}"),
 Translation(('SOUT',) : "south")]
stroke = Stroke(SOUT : ['S-', 'O-', '-U', '-T'])
old = []
new = [Action(prev_attach=True, prev_replace='south', text='So
uth', trailing_space=' ', word='south')]
# pass filter
english  = South
stroked  = ['KA*PD']
suggestions  = [('SOUT',), ('SO*UT',)]
phrase = ('South', ['KA*PD'], [('SOUT',), ('SO*UT',)])
english  = South
english  = South
translations  = [Translation(('PHROLG',) : "{PLOVER:TOGGLE}"),
 Translation(('SOUT',) : "south"), Translation(('KA*PD',) : "{
*-|}")]
stroke = Stroke(KA*PD : ['K-', 'A-', '*', '-P', '-D'])


# fun with advertise

old = [Action(text='add', trailing_space=' ', word='add')]
new = [Action(text='advertise', trailing_space=' ', word='adve
rtise')]
pass filter
english  = advertise
stroked  = ['AD', 'TAOEUS']
suggestions  = [('TEUZ',), ('TAOEUS',), ('AD', 'TAOEUS'), ('AD
', 'SRER', 'TAOEUZ')]
phrase = ('advertise', ['AD', 'TAOEUS'], [('TEUZ',), ('TAOEUS'
,), ('AD', 'TAOEUS'), ('AD', 'SRER', 'TAOEUZ')])
english  = advertise
stroked  = ['PHROLG', 'AD', 'TAOEUS']
suggestions  = [('TEUZ',), ('TAOEUS',), ('AD', 'TAOEUS'), ('AD
', 'SRER', 'TAOEUZ')]
phrase = ('advertise', ['PHROLG', 'AD', 'TAOEUS'], [('TEUZ',),
 ('TAOEUS',), ('AD', 'TAOEUS'), ('AD', 'SRER', 'TAOEUZ')])
translations  = [Translation(('PHROLG',) : "{PLOVER:TOGGLE}"),
 Translation(('AD', 'TAOEUS') : "advertise")]
stroke = Stroke(TAOEUS : ['T-', 'A-', 'O-', '-E', '-U', '-S'])


# annoying retrospective delete space

old = [Action(text='writer', trailing_space=' ', word='writer'
)]
new = [Action(next_attach=True, prev_attach=True, text='', wor
d='', word_is_finished=False), Action(prev_attach=True, text='
writer', trailing_space=' ', word='writer')]
pass filter
english  = typewriter
stroked  = ['TK-FPS']
suggestions  = [('TWRAOEUR',), ('TAOEUP', 'WREUR'), ('TAOEUP',
 'WRAOEURT'), ('TAOEUP', 'WREU', 'ER'), ('TAOEUP', 'WRAOEUT',
'ER'), ('TAOEUP', 'WRAOEUT', '*ER')]
phrase = ('typewriter', ['TK-FPS'], [('TWRAOEUR',), ('TAOEUP',
 'WREUR'), ('TAOEUP', 'WRAOEURT'), ('TAOEUP', 'WREU', 'ER'), (
'TAOEUP', 'WRAOEUT', 'ER'), ('TAOEUP', 'WRAOEUT', '*ER')])
english  = typewriter
stroked  = ['PHROLG', 'TK-FPS']
suggestions  = [('TWRAOEUR',), ('TAOEUP', 'WREUR'), ('TAOEUP',
 'WRAOEURT'), ('TAOEUP', 'WREU', 'ER'), ('TAOEUP', 'WRAOEUT',
'ER'), ('TAOEUP', 'WRAOEUT', '*ER')]
phrase = ('typewriter', ['PHROLG', 'TK-FPS'], [('TWRAOEUR',),
('TAOEUP', 'WREUR'), ('TAOEUP', 'WRAOEURT'), ('TAOEUP', 'WREU'
, 'ER'), ('TAOEUP', 'WRAOEUT', 'ER'), ('TAOEUP', 'WRAOEUT', '*
ER')])
translations  = [Translation(('PHROLG',) : "{PLOVER:TOGGLE}"),
 Translation(('TK-FPS',) : "type{^~|^}writer")]
stroke = Stroke(TK-FPS : ['T-', 'K-', '-F', '-P', '-S'])

